var ShortcutKeys = (function (exports) {
  'use strict';

  var Key;
  (function (Key) {
      Key["Shift"] = "shift";
      Key["Control"] = "control";
      Key["Command"] = "cmd";
      Key["Alt"] = "alt";
      Key["Space"] = " ";
      Key["Enter"] = "enter";
  })(Key || (Key = {}));

  const excludes = [Key.Control, Key.Shift, Key.Alt];
  const shortcutKeys = (element) => {
      const shortcutMap = {};
      const hasKey = (obj, key) => Object.prototype.hasOwnProperty.call(obj, key);
      const incrementUserAction = (e, options) => {
          const key = e.key.toLowerCase();
          let keys = [];
          if ((e.ctrlKey || e.shiftKey || e.altKey) && excludes.includes(key)) {
              return;
          }
          if (options.multiPlatform && (e.ctrlKey || e.metaKey))
              keys.push(Key.Control);
          else if (e.ctrlKey)
              keys.push(Key.Control);
          else if (e.metaKey)
              keys.push(Key.Command);
          if (e.shiftKey)
              keys.push(Key.Shift);
          if (e.altKey)
              keys.push(Key.Alt);
          if (key === Key.Space)
              keys.push("space");
          if (key === Key.Enter)
              keys.push(Key.Enter);
          if (key)
              keys.push(key);
          keys = keys.map((key) => key.trim()).filter(Boolean);
          const concatKeys = keys.join("+");
          const eventFound = shortcutMap[concatKeys];
          if (eventFound && options.prevent)
              e.preventDefault();
          return concatKeys;
      };
      const defaultOptions = {
          description: "",
          multiPlatform: true,
          prevent: true,
          eventType: "keydown",
      };
      const defaultNativeOptions = {
          capture: undefined,
          once: undefined,
          passive: undefined,
          signal: undefined,
      };
      const add = (shortcut, handler, options = defaultOptions, nativeOptions = defaultNativeOptions) => {
          const concatOptions = { ...defaultOptions, ...options };
          const concatNativeOptions = { ...defaultOptions, ...nativeOptions };
          const shortcuts = Array.isArray(shortcut) ? shortcut : [shortcut];
          shortcuts.forEach((shortcutItem) => {
              const key = shortcutItem.toLowerCase().trim();
              const target = (e) => incrementUserAction(e, options) === key
                  ? handler(e)
                  : undefined;
              shortcutMap[key] = {
                  nativeOptions: concatNativeOptions,
                  options: concatOptions,
                  target: target,
              };
              element.addEventListener(concatOptions.eventType, target, concatNativeOptions);
          });
      };
      const removeAll = () => Object.entries(shortcutMap).forEach(([_, { target, options, nativeOptions }]) => {
          element.removeEventListener(options.eventType, target, nativeOptions);
      });
      const remove = (shortcut) => {
          if (!hasKey(shortcutMap, shortcut))
              return;
          return element.removeEventListener(shortcutMap[shortcut].options.eventType, shortcutMap[shortcut].target);
      };
      const list = () => {
          return Object.entries(shortcutMap).map(([key, info]) => {
              return {
                  key,
                  info
              };
          });
      };
      return {
          /**
           * @description Add event to element
           * @param shortcut - Required - Shortcut to trigger action. Example: "control+h" or ["control+h", "control+shift+h"]
           * @param handler - Required - Action triggered when shortcut is triggered. Example: () => console.log("hello");
           * @param options - Optional - Extra settings. Read the documentation {@link "https://www.npmjs.com/package/shortcut-keys"}
           */
          add,
          /**
           * @description Remove exists event to element
           * @param shortcut - Required - Shortcut to trigger action. Example: "control+h" or ["control+h", "control+shift+h"]. When there is no data, all element events will be removed.
           */
          remove,
          /**
           * @description Remove all exists events
           */
          removeAll,
          /**
            * @description List all events of element
            * @returns List about shortcuts
            * @property {string} key - Shortcut to trigger action.
            * @property {Object} info - All information about the shortcut.
           */
          list,
      };
  };
  if (typeof window !== "undefined") {
      window.shortcutKeys = shortcutKeys;
  }

  exports.shortcutKeys = shortcutKeys;

  return exports;

})({});
